package org.insa.graphs.algorithm.shortestpath;

import static org.junit.Assert.*;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.insa.graphs.algorithm.ArcInspector;
import org.insa.graphs.model.Graph;
import org.insa.graphs.model.Path;
import org.insa.graphs.model.io.BinaryGraphReader;
import org.insa.graphs.model.io.BinaryPathReader;
import org.insa.graphs.model.io.GraphReader;
import org.insa.graphs.model.io.PathReader;
import org.junit.BeforeClass;
import org.junit.Test;

public class DijkstraAlgorithmTest {

    // TODO finish this
    public GraphReader reader;
    public Graph graph;
    public PathReader pathReader;
    public Path path;


    @BeforeClass
    public void init() {
        // Visit these directory to see the list of available files on Commetud.
        // When running with VSC, paths are relative to the BE_Graphes directory.
        final String mapName = "./Maps/insa.mapgr";
        final String pathName = "./Paths/path_fr31insa_rangueil_r2.path";
        System.out.println("Working Directory = " + System.getProperty("user.dir"));
        // Create a graph reader.
        try {
            final GraphReader reader = new BinaryGraphReader(
                    new DataInputStream(new BufferedInputStream(new FileInputStream(mapName))));

            // Read the graph. X
            this.graph = reader.read();
            // free resources
            reader.close();

            // Create a PathReader.
            final PathReader pathReader = new BinaryPathReader(new DataInputStream(new BufferedInputStream(new FileInputStream(pathName))));
            // free resources
            pathReader.close();

            // Read the path.
            this.path = pathReader.readPath(graph);
        }
        catch (FileNotFoundException e) {
            System.err.println("File not found: " + e.getMessage());
            fail("File not found: " + e.getMessage());
        }
        catch (IOException e ) {
            System.err.println("Error reading file: " + e.getMessage());
            fail("Error reading file: " + e.getMessage());
        }
    }

    @Test
    // TODO
    // fonction pour code au dessus
    // appeler constructeur dijkstra
    // donner le path à afficher en renvoyant le path de dijkstra dans path.
    // Chemin court => Bellman OK et vérifié. On compare les résultats obtenus par les 2 algos
    public void Test_Toulouse_court_chemin() {
        ArcInspector arcInspector;
        ShortestPathData data = new ShortestPathData(this.graph, this.path.getOrigin(), this.path.getDestination(), null);
        
        DijkstraAlgorithm dijkstra = new DijkstraAlgorithm(data);
        ShortestPathSolution dijk_path = dijkstra.doRun();

        BellmanFordAlgorithm bellman = new BellmanFordAlgorithm(data);
        ShortestPathSolution bell_path = bellman.doRun();
        assert(dijk_path.getPath() == bell_path.getPath());
    }

    // Long chemin => Bellman trop long, on compare dijkstra au chemin récupéré directement
    public void Test_Toulouse_long_chemin() {
        init();
        ArcInspector arcInspector;
        ShortestPathData data = new ShortestPathData(this.graph, this.path.getOrigin(), this.path.getDestination(), null);
        DijkstraAlgorithm dijkstra = new DijkstraAlgorithm(data);
        ShortestPathSolution dijk_path = dijkstra.doRun();

        assert(dijk_path.getPath() == this.path);
    }

    
}
